/*
 * Date  : Jan 12, 2011
 * Author: K.Jaensch, klausj@phonetik.uni-muenchen.de
 */
 
package ips.audio.coreaudio;

import java.io.File;
import java.net.URL;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.Hashtable;

import javax.sound.sampled.Mixer;
import javax.sound.sampled.Mixer.Info;
import javax.sound.sampled.spi.MixerProvider;

/**
 * @author K.Jaensch, klausj@phonetik.uni-muenchen.de
 *
 */

public class CoreAudioMixerProvider extends MixerProvider {
    
    final static boolean DEBUG = false;
    final static String NATIVE_LIB_LOCATION_KEY="ips.coreaudio.nativelib";
    public final static String SERVICE_DESCRIPTION_FILE_PATH="ipsk.audio.ajs.MixerProviderServiceDescriptor.xml";
    final static boolean LIST_PLUGINS=false;
    private final static String META_PATH="META-INF";
    private final static String META_SERVICES_PATH=META_PATH+"/services/";
    private boolean useAJSSuffix=false;
    
    //private static ArrayList<CoreAudioMixerInfo> infoList=null;
    private static ArrayList<CoreAudioMixer> mixers=null;
    static {
    	if(DEBUG)System.out.println("CoreAudio-Java... ");
        String dllLocation=null;
        try{
            dllLocation=System.getProperty(NATIVE_LIB_LOCATION_KEY);    
        }catch(SecurityException se){
            //OK no debug mode, continue
        }
        
        if (dllLocation!=null){
            if(DEBUG)System.out.println("CoreAudio-Java library loading ... ");
            File dsLib=new File(dllLocation);
            if(!dsLib.exists()){
                System.err.println("CoreAudioJavaSound library not found!");
            }
            System.load(dllLocation);
            
        }else{
        	try{
            System.loadLibrary("CoreAudioJavaSoundLib");
        	}catch (UnsatisfiedLinkError usle){
        		System.out.println("java.library-path: "+System.getProperty("java.library.path"));
        		usle.printStackTrace();
        	}
        }
        if (DEBUG)
            System.out.println("CoreAudio driver loaded");
        
    }
    
    public CoreAudioMixerProvider(){
    	super();
    	String ajsXmlServiceDescriptorPath=META_SERVICES_PATH+SERVICE_DESCRIPTION_FILE_PATH;
    	ClassLoader classLoader=getClass().getClassLoader();
    	if(classLoader!=null){
    		URL sdURL=classLoader.getResource(ajsXmlServiceDescriptorPath);
    		if(sdURL==null){
    			// if no XML descriptor found we are in standard JavaSound mode
    			// and need the suffix " (AJS)"
    			useAJSSuffix=true;
    			if(DEBUG)System.out.println(ajsXmlServiceDescriptorPath+" not found. Switching suffix on");
    		}
    	}
    }
    
    private native void enumerateCards();
   
   // private native void enumerateDevices(String cardId);
    
    
    protected void nextDevice(ByteBuffer audioDeviceID){
    	
    	CoreAudioMixer mixer=new CoreAudioMixer(audioDeviceID,useAJSSuffix);
    	mixers.add(mixer);
    }
  
    @Override
    public synchronized Mixer getMixer(Info info) {
       
        for(CoreAudioMixer m:mixers){
        	Mixer.Info mInfo=m.getMixerInfo();
        	if(mInfo.equals(info)){
        		return m;
        	}
        }
        throw new IllegalArgumentException();
    }

    @Override
    public synchronized Info[] getMixerInfo() {
        Info[] mixerInfos=null;
        
        if(mixers==null){
            mixers=new ArrayList<CoreAudioMixer>();
            enumerateCards();
        }
            
        ArrayList<Mixer.Info> infos=new ArrayList<Mixer.Info>();
        for(CoreAudioMixer m:mixers){
        	infos.add(m.getMixerInfo());
        }
       mixerInfos=infos.toArray(new Info[0]);
            
        
      return mixerInfos;
    }

    /**
     * Test 
     * @param args
     */
    public static void main(String[] args) {
        CoreAudioMixerProvider amp=new CoreAudioMixerProvider();
        Info[] infs=amp.getMixerInfo();
        for(Info info:infs){
        	System.out.println("Mixer: "+info.getName());
        }

    }

}
