package ips.audio.coreaudio;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.TargetDataLine;

public class CoreAudioTargetDataLine extends CoreAudioDataLine implements
		TargetDataLine {
	
	
//	private int writePos=0;
//	private int readPos=0;
//	
//	private FloatBuffer ringBufferFloat;
//	
//	private float maxValue;
//	private boolean signed=true;
//	private int sampleSize;
//	
//	private boolean bigEndian=true;
//	
//	private int physicalChannels;
//	private int channels;

	public CoreAudioTargetDataLine(CoreAudioMixer mixer,ByteBuffer dataline) {
		super(mixer,dataline);
		AudioFormat[] suppFormats=getSupportedFormats().toArray(new AudioFormat[0]);
	     try{
	    	 lineInfo=new DataLine.Info(TargetDataLine.class,suppFormats,-1,-1);
	     }catch(IllegalArgumentException iae){
	         if(DEBUG)System.out.println("Illegal data line ");  
	     }
	}

	
	


	
	
	

	/**
	 * Encodes float values into byte array.
	 * 
	 * @param f
	 *            normalized amplitude value (-1.0 ... 1.0)
	 * @param buf
	 *            target data buffer
	 * @param offset
	 *            offset in the buffer
	 */
	public void encodeValue(float f, byte[] buf, int offset) {
		long value = (long) (maxValue * f);
		if (signed) {
			if (bigEndian) {
				for (int i = sampleSize - 1; i > 0; i--) {
					
					buf[offset + i] = (byte) value;
					value = value >> 8;
				}
				buf[offset] = (byte) value;
			} else {
				for (int i = 0; i < sampleSize - 1; i++) {
					buf[offset + i] = (byte) value;
					value = value >> 8;
				}
				buf[offset + sampleSize - 1] = (byte) value;
			}
		}
	}
	
	private native int coreAudioBytesAvailable(ByteBuffer jdl);
	@Override
	public int available() {
		int caAvailBytes=coreAudioBytesAvailable(dataline);
		return (int)coreAudioByteCountToJsByteCount(caAvailBytes);

	}
	
	@Override
	public long getLongFramePosition() {
		long coreAudioFloatBytePosition=getWritePos(dataline);	
		return coreAudioByteCountToJsByteCount(coreAudioFloatBytePosition);
	}
	
	private native void addToReadPos(ByteBuffer dataline,int read);
	
	@Override
	public int read(byte[] arg0, int arg1, int arg2) {
		
		//int r=read(dataline,arg0,arg1,arg2);
		long readPos=getReadPos(dataline);
		int avail=0;
		do{
			avail=available();
			if(avail==0){
				try {
					Thread.sleep(10);
				} catch (InterruptedException e) {
					// OK
				}
			}
		}while(avail<=0 && active);
		int toRead1=arg2;
		if(toRead1 > avail){
			toRead1=avail;
		}
		long bufReadPos=readPos%coreAudioBufferSize;
		//long rbToRead1=toRead1*COREAUDIO_FLOAT_SAMPLESIZE/sampleSize;
		long rbToRead1=jsByteCountToCoreAudioByteCount(toRead1);
		if(bufReadPos+rbToRead1 > coreAudioBufferSize){
			rbToRead1=coreAudioBufferSize-bufReadPos;
		}
		long floatStartIndex=bufReadPos/COREAUDIO_FLOAT_SAMPLESIZE;
		long floatLen=rbToRead1/COREAUDIO_FLOAT_SAMPLESIZE;
		int offset=arg1;
		
		for(long i=0;i<floatLen;i++){
			int currPCh=(int)i%physicalChannels;
			if(currPCh<channels){
				float v=ringBufferFloat.get((int) (floatStartIndex+i));
				encodeValue(v, arg0, offset);
				offset+=sampleSize;
			}
			bufReadPos+=COREAUDIO_FLOAT_SAMPLESIZE;
		}

		if(rbToRead1>0){
			addToReadPos(dataline,(int)rbToRead1);
		}
		return offset-arg1;
	}

	@Override
	public void drain() {
		// Do nothing (drain makes no sense in capture mode)
	}

	private native void flush(ByteBuffer dataline);
	@Override
	public void flush() {
		flush(dataline);
	}

	

}
